/* ---------------------------------------- */
/* SPI functions
 *  based  stm32f4xx_spi.h @ STMicroelectronics
 */

#include "stm32f4xx.h"

#define CR1_CLEAR_MASK                  ((uint32_t)0xFCFFFEFF)
#define SPI_Direction_2Lines_FullDuplex ((uint16_t)0x0000)
#define SPI_Direction_2Lines_RxOnly     ((uint16_t)0x0400)
#define SPI_Direction_1Line_Rx          ((uint16_t)0x8000)
#define SPI_Direction_1Line_Tx          ((uint16_t)0xC000)
#define SPI_Mode_Master                 ((uint16_t)0x0104)
#define SPI_Mode_Slave                  ((uint16_t)0x0000)
#define SPI_DataSize_16b                ((uint16_t)0x0800)
#define SPI_DataSize_8b                 ((uint16_t)0x0000)
#define SPI_CPOL_Low                    ((uint16_t)0x0000)
#define SPI_CPOL_High                   ((uint16_t)0x0002)
#define SPI_CPHA_1Edge                  ((uint16_t)0x0000)
#define SPI_CPHA_2Edge                  ((uint16_t)0x0001)
#define SPI_NSS_Soft                    ((uint16_t)0x0200)
#define SPI_NSS_Hard                    ((uint16_t)0x0000)
#define SPI_BaudRatePrescaler_2         ((uint16_t)0x0000)
#define SPI_BaudRatePrescaler_4         ((uint16_t)0x0008)
#define SPI_BaudRatePrescaler_8         ((uint16_t)0x0010)
#define SPI_BaudRatePrescaler_16        ((uint16_t)0x0018)
#define SPI_BaudRatePrescaler_32        ((uint16_t)0x0020)
#define SPI_BaudRatePrescaler_64        ((uint16_t)0x0028)
#define SPI_BaudRatePrescaler_128       ((uint16_t)0x0030)
#define SPI_BaudRatePrescaler_256       ((uint16_t)0x0038)
#define SPI_FirstBit_MSB                ((uint16_t)0x0000)
#define SPI_FirstBit_LSB                ((uint16_t)0x0080)

typedef enum
{ 
  GPIO_Mode_IN   = 0x00, /*!< GPIO Input Mode */
  GPIO_Mode_OUT  = 0x01, /*!< GPIO Output Mode */
  GPIO_Mode_AF   = 0x02, /*!< GPIO Alternate function Mode */
  GPIO_Mode_AN   = 0x03  /*!< GPIO Analog Mode */
}GPIOMode_TypeDef;

typedef enum
{ 
  GPIO_OType_PP = 0x00,
  GPIO_OType_OD = 0x01
}GPIOOType_TypeDef;

typedef enum
{ 
  GPIO_PuPd_NOPULL = 0x00,
  GPIO_PuPd_UP     = 0x01,
  GPIO_PuPd_DOWN   = 0x02
}GPIOPuPd_TypeDef;

typedef enum
{ 
  GPIO_Speed_2MHz   = 0x00, /*!< Low speed */
  GPIO_Speed_25MHz  = 0x01, /*!< Medium speed */
  GPIO_Speed_50MHz  = 0x02, /*!< Fast speed */
  GPIO_Speed_100MHz = 0x03  /*!< High speed on 30 pF (80 MHz Output max speed on 15 pF) */
}GPIOSpeed_TypeDef;


void  set_pin_mode( GPIO_TypeDef* GPIOx, uint32_t  pinpos )
{
    GPIOx->MODER &= ~(GPIO_MODER_MODER0 << (pinpos * 2));
    GPIOx->MODER |= (((uint32_t)GPIO_Mode_AF) << (pinpos * 2));

    /* -- for GPIO_Mode_OUT || GPIO_Mode_AF --- */
    /* Speed mode configuration */
    GPIOx->OSPEEDR &= ~(GPIO_OSPEEDER_OSPEEDR0 << (pinpos * 2));
    GPIOx->OSPEEDR |= ((uint32_t)(GPIO_Speed_50MHz) << (pinpos * 2));

    /* Output mode configuration*/
    GPIOx->OTYPER &= ~((GPIO_OTYPER_OT_0) << ((uint16_t)pinpos)) ;
    GPIOx->OTYPER |= (uint16_t)(GPIO_OType_PP << ((uint16_t)pinpos));
    /* ---------------------------------------- */

    /* Pull-up Pull down resistor configuration*/
    GPIOx->PUPDR &= ~(GPIO_PUPDR_PUPDR0 << ((uint16_t)pinpos * 2));
    //GPIOx->PUPDR |= (((uint32_t)GPIO_PuPd_DOWN) << (pinpos * 2));
    GPIOx->PUPDR |= (((uint32_t)GPIO_PuPd_UP) << (pinpos * 2));
}


SPI_TypeDef  *SPIx ;


uint16_t  xspi_send( uint16_t data )
{
    /* wait for send mode */
    while( (SPIx->SR & SPI_SR_TXE) == 0 ) ;

    SPIx->DR = data ;

    /* wait for recv data */
    while( (SPIx->SR & SPI_SR_RXNE) == 0 ) ;

    return  SPIx->DR ;
}


uint16_t  xspi_recv( void )
{
    /* wait for send mode */
    while( (SPIx->SR & SPI_SR_TXE) == 0 ) ;
    SPIx->DR = (uint16_t)0x00ff ;

    /* wait for recv data */
    while( (SPIx->SR & SPI_SR_RXNE) == 0 ) ;

    return  SPIx->DR ;
}


/** for SD card **/
void  xspi_set_cs_sd( int flag )
{
    if( flag != 0 ) {
        GPIOB->BSRR = 0x0040 ;       /* CS:1 */
    } else {
        GPIOB->BSRR = 0x0040 << 16 ; /* CS:0 */
    }
}


/** for LCD **/
void  xspi_set_cs_lcd( int flag )
{
    if( flag != 0 ) {
        GPIOB->BSRR = 0x0020 ;       /* CS:1 */
    } else {
        GPIOB->BSRR = 0x0020 << 16 ; /* CS:0 */
    }
}


/** for TouchPannel **/
void  xspi_set_cs_tp( int flag )
{
    if( flag != 0 ) {
        GPIOB->BSRR = 0x0010 ;       /* CS:1 */
    } else {
        GPIOB->BSRR = 0x0010 << 16 ; /* CS:0 */
    }
}


/** for NFC / RFID **/
void  xspi_set_cs_nfc( int flag )
{
    if( flag != 0 ) {
        GPIOB->BSRR = 0x0008 ;       /* CS:1 */
    } else {
        GPIOB->BSRR = 0x0008 << 16 ; /* CS:0 */
    }
}


void  xspi_mode_msb( void )
{
    uint16_t  tmpreg = 0;

    /* Get the SPIx CR1 value */
    tmpreg = SPIx->CR1;
    /* Clear BIDIMode, BIDIOE, RxONLY, SSM, SSI, LSBFirst, BR, MSTR, CPOL and CPHA bits */
    tmpreg &= CR1_CLEAR_MASK;
    tmpreg |= ( uint16_t)
              ( SPI_Direction_2Lines_FullDuplex
              | SPI_Mode_Master
              | SPI_DataSize_8b
              | SPI_CPOL_Low
              | SPI_CPHA_1Edge
              | SPI_NSS_Soft
              | SPI_BaudRatePrescaler_16
//              | SPI_BaudRatePrescaler_32
              | SPI_FirstBit_MSB
              ) ;

    /* Write to SPIx CR1 */
    SPIx->CR1 = tmpreg;

    /* Activate the SPI mode (Reset I2SMOD bit in I2SCFGR register) */
    SPIx->I2SCFGR &= ~((uint16_t)SPI_I2SCFGR_I2SMOD);

    /*---------------------------- SPIx CRCPOLY Configuration --------------------*/
    /* Write to SPIx CRCPOLY */
    SPIx->CRCPR = 7;

    /* SPI enable */
    SPIx->CR1 |= SPI_CR1_SPE;
}


void  xspi_mode_lsb( void )
{
    uint16_t  tmpreg = 0;

    /* Get the SPIx CR1 value */
    tmpreg = SPIx->CR1;
    /* Clear BIDIMode, BIDIOE, RxONLY, SSM, SSI, LSBFirst, BR, MSTR, CPOL and CPHA bits */
    tmpreg &= CR1_CLEAR_MASK;
    tmpreg |= ( uint16_t)
              ( SPI_Direction_2Lines_FullDuplex
              | SPI_Mode_Master
              | SPI_DataSize_8b
              | SPI_CPOL_Low
//              | SPI_CPOL_High
              | SPI_CPHA_1Edge
//              | SPI_CPHA_2Edge
              | SPI_NSS_Soft
//              | SPI_BaudRatePrescaler_16
              | SPI_BaudRatePrescaler_32
              | SPI_FirstBit_LSB
              ) ;

    /* Write to SPIx CR1 */
    SPIx->CR1 = tmpreg;

    /* Activate the SPI mode (Reset I2SMOD bit in I2SCFGR register) */
    SPIx->I2SCFGR &= ~((uint16_t)SPI_I2SCFGR_I2SMOD);

    /*---------------------------- SPIx CRCPOLY Configuration --------------------*/
    /* Write to SPIx CRCPOLY */
    SPIx->CRCPR = 7;

    /* SPI enable */
    SPIx->CR1 |= SPI_CR1_SPE;
}


void  xspi_init( void )
{
    SPIx = SPI1 ;

    /* set clock enable */
    RCC->AHB1ENR |= (  RCC_AHB1ENR_GPIOAEN
                     | RCC_AHB1ENR_GPIOBEN
                    ) ;
    RCC->APB2ENR |= RCC_APB2ENR_SPI1EN ;

    /** for CS_LCD:PB5 **/
    GPIOB->MODER   &= ~(0x00000c00);
    GPIOB->MODER   |=  (0x00000400);
    GPIOB->OTYPER  &= ~(0x0020);
    GPIOB->OSPEEDR &= ~(0x00000c00);
    GPIOB->OSPEEDR |=  (0x00000800);
    GPIOB->PUPDR   &= ~(0x00000c00);
    GPIOB->PUPDR   |=  (0x00000400);
    GPIOB->BSRR = 0x0020 ;       /* CS:1 */

    set_pin_mode( GPIOA,  5 ) ; /* SPI1 : SCK */
    set_pin_mode( GPIOA,  6 ) ; /* SPI1 : MISO */
    set_pin_mode( GPIOA,  7 ) ; /* SPI1 : MOSI */
    GPIOA->AFR[0] &= ~(0xfff00000 ) ;
    GPIOA->AFR[0] |=  (0x55500000 ) ; /* set AF5 */

    xspi_mode_msb();
}

/* ---------------------------------------- */

