/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: sample1.c 2416 2012-09-07 08:06:20Z ertl-hiro $
 */

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"
#include "dev_head_out.h"


int  dev_btn_status;

void  WaitMsec( int );


void  dev_gpio_out_init( void )
{
    /* set clock enable */
    RCC->AHB1ENR |= ( RCC_AHB1ENR_GPIOCEN
                    ) ;

    /** for BTN:PC13 **/
    GPIOC->MODER   &= ~(0x0c000000);
    GPIOC->OTYPER  |=  (0x2000);
    GPIOC->PUPDR   &= ~(0x0c000000);
    GPIOC->PUPDR   |=  (0x04000000);
}


int  dev_get_btn_stat( void )
{
    int  stat = 0 ;
    unsigned int  data ;

    data = GPIOC->IDR ;
    if( (data & (1 << 13)) == 0 ) {
        stat |= 0x01 ;
    }

    return( stat ) ;
}


Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
	if (ercd < 0) {
		t_perror(LOG_ERROR, file, line, expr, ercd);
	}
}

#define	SVC_PERROR(expr)	svc_perror(__FILE__, __LINE__, #expr, (expr))


void tex_routine(TEXPTN texptn, intptr_t exinf)
{
	int_t	tskno = (int_t) exinf;

	syslog(LOG_NOTICE, "task%d receives exception 0x%04x.", tskno, texptn);

	if ((texptn & 0x8000U) != 0U) {
		syslog(LOG_INFO, "#%d#ext_tsk()", tskno);
		SVC_PERROR(ext_tsk());
		assert(0);
	}
}


/*
 *  CPU㳰ϥɥ
 */
#ifdef CPUEXC1

void
cpuexc_handler(void *p_excinf)
{
	ID		tskid;

	syslog(LOG_NOTICE, "CPU exception handler (p_excinf = %08p).", p_excinf);
	if (sns_ctx() != true) {
		syslog(LOG_WARNING,
					"sns_ctx() is not true in CPU exception handler.");
	}
	if (sns_dpn() != true) {
		syslog(LOG_WARNING,
					"sns_dpn() is not true in CPU exception handler.");
	}
	syslog(LOG_INFO, "sns_loc = %d sns_dsp = %d sns_tex = %d",
									sns_loc(), sns_dsp(), sns_tex());
	syslog(LOG_INFO, "xsns_dpn = %d xsns_xpn = %d",
									xsns_dpn(p_excinf), xsns_xpn(p_excinf));

	if (xsns_xpn(p_excinf)) {
		syslog(LOG_NOTICE, "Sample program ends with exception.");
		SVC_PERROR(ext_ker());
		assert(0);
	}

	SVC_PERROR(iget_tid(&tskid));
	SVC_PERROR(iras_tex(tskid, 0x8000U));
}

#endif /* CPUEXC1 */

/*
 *  ϥɥ
 *
 *  HIGH_PRIORITYMID_PRIORITYLOW_PRIORITY γͥ٤Υǥ塼
 *  ž롥
 */
void cyclic_handler(intptr_t exinf)
{
}

/*
 *  顼ϥɥ
 *
 *  HIGH_PRIORITYMID_PRIORITYLOW_PRIORITY γͥ٤Υǥ塼
 *  ž롥
 */
void alarm_handler(intptr_t exinf)
{
}


#define  DEF_TASK_WAIT       30
#define  DEF_TASK_NORM_ACT    2
#define  DEF_TASK_LONG_ACT   10


void task01(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    dev_btn_status = dev_get_btn_stat();
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task02(intptr_t exinf)
{
  int  act_time;

  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    if( dev_btn_status ) {
      act_time = 20;
    } else {
      act_time = DEF_TASK_NORM_ACT;
    }
    WaitMsec( act_time );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task03(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task04(intptr_t exinf)
{
  int  act_time;

  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    if( dev_btn_status ) {
      act_time = 20;
    } else {
      act_time = DEF_TASK_NORM_ACT;
    }
    WaitMsec( act_time );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task05(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task06(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task07(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task08(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task09(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


void task10(intptr_t exinf)
{
  syslog(LOG_NOTICE, "%s(exinf = %d).", __FUNCTION__, (int_t) exinf);

  while( 1 ) {
    WaitMsec( DEF_TASK_NORM_ACT );
    tslp_tsk( DEF_TASK_WAIT );
  }
}


extern  int  _kernel_tmax_tskid;


void main_task(intptr_t exinf)
{
	ER_UINT	ercd;

	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	ercd = serial_opn_por(TASK_PORTID);
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(TASK_PORTID,
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	dev_btn_status = 0;
	dev_gpio_out_init();

	SVC_PERROR(act_tsk(TASK01));
	SVC_PERROR(act_tsk(TASK02));
	SVC_PERROR(act_tsk(TASK03));
	SVC_PERROR(act_tsk(TASK04));
	SVC_PERROR(act_tsk(TASK05));
	SVC_PERROR(act_tsk(TASK06));
	SVC_PERROR(act_tsk(TASK07));
	SVC_PERROR(act_tsk(TASK08));
	SVC_PERROR(act_tsk(TASK09));
	SVC_PERROR(act_tsk(TASK10));

	SVC_PERROR(sta_cyc(CYCHDR1));

	syslog(LOG_NOTICE, "%s() _kernel_tmax_tskid=%d", __FUNCTION__, _kernel_tmax_tskid);

	/*
 	 *  ᥤ롼
	 */
	do {
          tslp_tsk( 100 );
          WaitMsec( 2 );
	} while( 1 );

	syslog(LOG_NOTICE, "Sample program ends.");
	SVC_PERROR(ext_ker());
	assert(0);
}
